/* $Id$

This file is part of NetVis
Copyright @ 2008 Mikko Rasa, Mikkosoft Productions
Distributed unter the GPL
*/

#include <cmath>
#include <msp/gl/immediate.h>
#include <msp/gl/matrix.h>
#include <msp/gl/transform.h>
#include <msp/time/units.h>
#include "host.h"
#include "packet.h"

using namespace Msp;

Packet::Packet(const Host &s, const Host *d, const GL::Color &c, unsigned i):
	src(s),
	dest(d),
	color(c),
	x(0),
	size(cbrt(i)*1.5),
	angle(rand()*M_PI*2/RAND_MAX),
	rspeed(rand()*M_PI/RAND_MAX-M_PI/2)
{ }

void Packet::tick(const Time::TimeDelta &dt)
{
	double f=dt/Time::sec;
	x+=f;
	if(x>1)
		x=1;
	angle+=rspeed*f;
	if(angle<0)
		angle+=M_PI*2;
	if(angle>=M_PI*2)
		angle-=M_PI;
}

void Packet::render(GL::PrimitiveBuilder &bld) const
{
	if(x<0)
		return;
	if(!src.get_active() || (dest && !dest->get_active()))
		return;

	const Vector2 &spos=src.get_position();

	if(dest)
	{
		const Vector2 &dpos=dest->get_position();
		Vector2 pos(spos.x*(1-x)+dpos.x*x, spos.y*(1-x)+dpos.y*x);
		Vector2 corner(cos(angle)*size, sin(angle)*size);

		if(bld.get_type()!=GL::QUADS)
		{
			bld.end();
			bld.begin(GL::QUADS);
		}
		bld.color(color.r, color.g, color.b, color.a);
		bld.vertex(pos.x+corner.x, pos.y+corner.y);
		bld.vertex(pos.x-corner.y, pos.y+corner.x);
		bld.vertex(pos.x-corner.x, pos.y-corner.y);
		bld.vertex(pos.x+corner.y, pos.y-corner.x);
	}
	else
	{
		bld.end();
		bld.begin(GL::TRIANGLE_FAN);
		bld.color(color.r, color.g, color.b, color.a*(1-x));
		bld.vertex(spos.x, spos.y);
		for(unsigned i=0; i<=24; ++i)
		{
			float a=i*M_PI/12;
			bld.vertex(spos.x+cos(a)*x*200, spos.y+sin(a)*x*200);
		}
	}
}
